/**
 * @copyright   2016-2018, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
import React from 'react';
import { MatchCallback, MatchResponse, Props } from './types';
export declare type MatcherFactory = (match: string, props: Props) => React.ReactNode;
export interface MatcherInterface {
    inverseName: string;
    propName: string;
    asTag(): string;
    createElement(match: string, props?: Props): React.ReactNode;
    match(value: string): MatchResponse | null;
    onBeforeParse(content: string, props: Props): string;
    onAfterParse(content: React.ReactNode[], props: Props): React.ReactNode[];
}
export default class Matcher<T extends {} = {}> implements MatcherInterface {
    options: T;
    propName: string;
    inverseName: string;
    factory: MatcherFactory | null;
    constructor(name: string, options?: Partial<T>, factory?: MatcherFactory | null);
    /**
     * Attempts to create a React element using a custom user provided factory,
     * or the default matcher factory.
     */
    createElement(match: string, props?: Props): React.ReactNode;
    /**
     * Replace the match with a React element based on the matched token and optional props.
     */
    replaceWith(match: string, props?: Props): React.ReactNode;
    /**
     * Defines the HTML tag name that the resulting React element will be.
     */
    asTag(): string;
    /**
     * Attempt to match against the defined string.
     * Return `null` if no match found, else return the `match`
     * and any optional props to pass along.
     */
    match(string: string): MatchResponse | null;
    /**
     * Trigger the actual pattern match and package the matched
     * response through a callback.
     */
    doMatch(string: string, pattern: string | RegExp, callback: MatchCallback): MatchResponse | null;
    /**
     * Callback triggered before parsing.
     */
    onBeforeParse(content: string, props: Props): string;
    /**
     * Callback triggered after parsing.
     */
    onAfterParse(content: React.ReactNode[], props: Props): React.ReactNode[];
}
